==> ddrilling-1.0.8.tgz: package/README.md <==
# About Directional Drilling
  In the early days of land drilling most wells were drilled vertically, straight down into the reservoir.Although these  wells were 
  considered to be vertical,thhey rarely were Some deviation in a wellbore will always occur,due to formation effects and bending
  of the drillstring. The first recorded instance of a well being deliberately drilled along a deviated course was in California in 1930
  This well was drilled to exploit a reservoir which was beyond the shoreline underneath the Pacific Ocean. It had been practice to 
  build jetties out into the ocean and build the drilling rig on the jetty
    
  However, this became prohibitively expensive and the technique of drilling deviated well was developed. Since then many new 
  techniques and special tools have been introduced to control the pathe of the wellbore. An operating company usually hires 
  a directional drilling service company to :
    
  * Provide expertise in planning the well.
  * Supply special tools.
  * To provide onsite assistance when operating the tools
    
  The operator may also hire a surveying company to measure the inclination and direction of the well as drilling proceeds
    
## Applications of Driectional Drilling
   Directional drilling can generally be defined as the science and act of drilling a wellbore along a predetermined trajectory
   to intersect a designated subsurface taget.(It has both vertical and deviation  component).

## Applications
* ###### Relief Wells
    Directional Drilling techniques are used to drill relief wells in order to kill blowouts.Relief wells are deviated to pass
    as close as possible to the uncontrolled well.Heavy mud is pumped into the reservoir to overcome the pressure and bring
    the wild well under control.

* ###### Controlling Vertical Wells
    Driectional Drilling techniques are used to straighten crooked holes. When deviation occurs in a well which is supposed to be
    vertical, various techniques can be used to bring the well back to vertical.This was one of the earlies application of directional
    drilling.

* ###### Sidetracking
    Sidetracking out of an existing wellbore is another application of directional drilling. This is done to bypass an obstruction
    (fish) in the original wellbore, to explore the extent of producing zone in a certain sector of a field, or to sidetrack a dry hole
    to a hole more promising target.

* ###### Inaccessible locations
    Directional wells are often drilled because the surface location,directly above the reservoir is inaccessible, either because
    of natural of man-made obstacles.

* ###### Fault Drilling
    Directional wells are also build to avoid drilling a vertical well through a steeply inclined fault plane which could slip and
    shear the casing.

* ###### Salt Dome Drilling
    Directional drilling programs are sometimes used to overcome the problems of drilling near salt domes.Instead of drilling through
    the salt, the well is drilled at one side of the dome and is then deviated around and underneath the overhanging cap.

* ###### Shoreline Drilling
    In the case where a reservoir lies offshore but quite close to land, the mose economical way to exploit the reservoir may be to
    drill directional wells from a land rig on the coast.Although it is not a new concept, one type of directional drilling, horiztonal 
    drilling is the fastest growing branch of drilling with major advances occuring in tools and techniques.

#   Depth Reference and Geographical Reference Systems
   The trajectory of a deviated well must be carefully planned so that the most efficient trajectory is used to drill between the rig
   and target location and ensure that the well is drilled for the least amount of money possible. When planning, and subsequently
   drilling the well, the position of all points along the wellpath and therefore the trajectory of the well must be considered in
   three dimensions.This means that the position of all points on the trajectory must be expessed with respect to a three dimensional
   reference system.The three dimensional system that is used to define the position of a particular point along the wellpath is:
    
   *   The vertical depth of the point below a particular reference point
   *   The horizontal distance traversed from the wellhead inn a Northerly direction
   *   The distance traversed from the well head in an Easterly direction
    
   The depth of a particular point in the wellpath is expressed in feet(or metres) vertically below a reference(datum) point and the
   Northerly and Easterly displacement of the point is expressed in feet(or metres) horizontally from the wellhead

##   Depth Reference Systems
   There are a number of datum systems used in the depth reference systems. The datum systems which are most widely used are
    
   * Mean Sea Level,MSL
   * Rotary Table Elevation,RTE
   * 20 Wellhead housing

   The Mean Sea Level,MSL is a permanent, national and well documented datum whereas datum such as the Rotary Table Elevation,RTE 
   only exists when the drilling rig is on site.The top of the 20 Wellhead Housing is only available when the wellhead  housing
   has been installed and will be removed when the well is abandoned. Hence,since the only permanent datum is the MSL,
   (the rig will be removed and the wellhead mya be removed on abandonment) the distance between the MSL and the rotary table
   on the drillfloor and the MSL and the wellhead housing must be measured and recorded carefully on the well survey documents.
   The elevation of the rotary table above the MSL will be measured when the drilling rig is placed over the drilling location.

   The depths of the formations to be penetrated are generally referenced by the geologist and reservoir engineers, to MSL since
   the Rotary Table Elevation will not be known until the driling rig is in place.In most drilling operations the Rotary Table
   Elevation(RTE) is used as the working depth reference since it is relatively simple, for the driller for instance, to measure
   depths relative to this point. The elevation of the (RTE) is also referred to as Derrick Floor ELevation(DFE).Depths measured
   from these references are often called depths below rotary table(BRT) or below derrick floor(BDF).The top of the kelly bushing
   is also used as a datum fo depth measurements, in this case the depths are referred to as depths below rotary kelly bushing(RKB)

   The depth of any point in the wellpath can be expressed in terms of the Along Hole Depth(AHD) and the True Vertical Depth(TVD) of
   the point below the reference datum. The AHD is the depth of a point from the surface reference point,measured along the trajectory
   of the borehole. Whereas the TVD is the vertical depth of the point below the reference point. The AHD will therefore always be
   greater than the TVD in a deviated well. Since there is no direct way of measuring the TVD, it must be calculated from the 
   information gathered when surveying the well.

###### Parameters Defining the WellPath
   There are three specific parameters which must be considered when planning trajectories of wells.These parameters combine to 
   define the trajectory of the well and are the
    
    * Kick-Off Point
    * Buildup and Drop off Rate
    * Tangent Angle of the Well

## The Kickoff Point(KOP)
   The kick off point is the along hole measured depth at which a change in inclination of the well is initiated and the well is 
   orientation in a particular direction(in terms of North,South,East and West). In general the most distant targets have the shallowest
   KOPs in order to reduce the incliation of the tangent section of the well.It is generally easier to kick of a well in shallow
   formations than in deep formations.The kick-off point should be initiated in formations which are stable and not likely to cause
   drilling problems, such as unconsolidated clays.

## Buildup Rate(BUR) and Drop Off Rate(DOR)
   The build up rate and drop off rate(in degrees of inclination)are the rates at which the well deviates from the vertical
   (usually measured in degrees per 100ft drilled).The build-up rate is chosen on the basis of drilling experience in the location
   and the tools available, but rates between 1 degree and 3 degree per 100ft of hole drilled are most common in conventional wells.
   Since the build up and drop off rates are constant, these sections of the well, by definition, form the arc of a circle.Buildup
   rates in excess of 3 degrees per 100ft are termed doglegs when drilling conventional deviated wells with conventional drilling
   equipment.The build up rate is often termed the dogleg severity.

## Tangent(or Drift) Angle
   The tangent angle(or drift angle) is the inclination(in degrees from the vertical) of the long straight section of the well is
   termed the tangent section because it forms a tangent to the arc formed by the build up section of the well.The tangent angle will
   generally be between 10 and 60 degrees since it is difficult to control the trajectory of the well at angles below 10 degrees and
   it is difficult to run wireline tools into wells at anglesof greater than 60 degrees.

#   Defining the Points on the Wellpath
   Having fixed the target and the rig position, the next stage is to plan the geometrical profile of the well to reach the target.
   The most common well trajectory is the build and hold profile, which consists of 3-sections vertical, buildup and tangent.The 
   trajectory of the wellbore can be plotted when the following points have been defined:

    *   KOP --(selected by designer)
    *   TVD[True vertical depth] and horizontal displacement of the end of the build up section.
    *   TVD and horizontal displacement of the target(defined by the position of the righ and target).

Since the driller will only be able to determine the along hole depth of the well the following information will also be required.

    * AHD of the KOP(same as TVD of KOP)
    * Build up rate for the build up section(selected by the designer)
    * Direction in which the well is to be drilled after the KOP in degees from the
        - North(defined by psitoin of the rig target),
        - AHD at which the build up stops and the tangent section commences and 
        - AHD of the target.

These depths and distances can be defined by a simple geometrical analysis of the well trajectory.

## npm package for directional drilling problems

Interface:

    build_UpAngleFxn(): number;

    horizontal_DisplacementFxn(): number;

    total_AngleYFxn(): number;

    total_AngleXplusYFxn(): number;

    measured_DepthAtEndOfBuildSectionFxn(): number;

    trueVertical_DepthAtEndOfBuildFxn(): number;

    horizontal_DeviationAtEndOfBuildUpFxn(): number;
    
    total_MeasuredDepthFxn(): number;

    toDegreesFxn(radians: number): number;

    toRadiansFxn(degrees: number): number;
    
    
## using Classes and Methods

   # TYPESCRIPT

    import { DirectionalDrilling } from "ddrilling";

    const inputData : InitalizeDrillingData = {
        buildUpAngle: 2,
        horizontalDisplacement: 3000,
        kickOffPoint: 2000,
        trueVerticalDepth: 10000
    }

    const drilling = new DirectionalDrilling(inputData);
    console.log(ddrilling.computationalResult());

    OUTPUT

    Directional Drilling Compuational Results
    -------------------------------------------
        Radius      => 2864.7889756541163
        Angle X     => 0.9682854378192228
        Angle Y     => 20.980333796791758
        Angle (X+Y) => 21.94861923461098
        Measured Depth At the End of Build => 3097.430961730549
        Horizontal Deviation at End of Build => 207.64158017340878
        Total Measured Depth => 10568.12478132156

    
   # JAVASCRIPT
   
    var ddrilling_1 = require("ddrilling");

     const inputData : InitalizeDrillingData = {
        buildUpAngle: 2,
        horizontalDisplacement: 3000,
        kickOffPoint: 2000,
        trueVerticalDepth: 10000
    }

    var drilling = new ddrilling_1.DirectionalDrilling(inputData);
    console.log(ddrilling.computationalResult())

     OUTPUT

    Directional Drilling Compuational Results
    -------------------------------------------
        Radius      => 2864.7889756541163
        Angle X     => 0.9682854378192228
        Angle Y     => 20.980333796791758
        Angle (X+Y) => 21.94861923461098
        Measured Depth At the End of Build => 3097.430961730549
        Horizontal Deviation at End of Build => 207.64158017340878
        Total Measured Depth => 10568.12478132156
    
   
###### sample Question used for the above implementation

    Given a planning procedure for the build and hold trajectory calculate the drift angle and the true vertical depth,
    given the following basic data:

    Basic Data
    Kick Off Point = 2000
    True Vertical depth = 10000
    Horizontal Displacement from Target = 3000
    BuildUpRate = 2degrees/100ft

==> ddrilling-1.0.11.tgz: package/README.md <==
# About Directional Drilling
  In the early days of land drilling most wells were drilled vertically, straight down into the reservoir.Although these  wells were 
  considered to be vertical,thhey rarely were Some deviation in a wellbore will always occur,due to formation effects and bending
  of the drillstring. The first recorded instance of a well being deliberately drilled along a deviated course was in California in 1930
  This well was drilled to exploit a reservoir which was beyond the shoreline underneath the Pacific Ocean. It had been practice to 
  build jetties out into the ocean and build the drilling rig on the jetty
    
  However, this became prohibitively expensive and the technique of drilling deviated well was developed. Since then many new 
  techniques and special tools have been introduced to control the pathe of the wellbore. An operating company usually hires 
  a directional drilling service company to :
    
  * Provide expertise in planning the well.
  * Supply special tools.
  * To provide onsite assistance when operating the tools
    
  The operator may also hire a surveying company to measure the inclination and direction of the well as drilling proceeds
    
## Applications of Driectional Drilling
   Directional drilling can generally be defined as the science and act of drilling a wellbore along a predetermined trajectory
   to intersect a designated subsurface taget.(It has both vertical and deviation  component).

## Applications
* ###### Relief Wells
    Directional Drilling techniques are used to drill relief wells in order to kill blowouts.Relief wells are deviated to pass
    as close as possible to the uncontrolled well.Heavy mud is pumped into the reservoir to overcome the pressure and bring
    the wild well under control.

* ###### Controlling Vertical Wells
    Driectional Drilling techniques are used to straighten crooked holes. When deviation occurs in a well which is supposed to be
    vertical, various techniques can be used to bring the well back to vertical.This was one of the earlies application of directional
    drilling.

* ###### Sidetracking
    Sidetracking out of an existing wellbore is another application of directional drilling. This is done to bypass an obstruction
    (fish) in the original wellbore, to explore the extent of producing zone in a certain sector of a field, or to sidetrack a dry hole
    to a hole more promising target.

* ###### Inaccessible locations
    Directional wells are often drilled because the surface location,directly above the reservoir is inaccessible, either because
    of natural of man-made obstacles.

* ###### Fault Drilling
    Directional wells are also build to avoid drilling a vertical well through a steeply inclined fault plane which could slip and
    shear the casing.

* ###### Salt Dome Drilling
    Directional drilling programs are sometimes used to overcome the problems of drilling near salt domes.Instead of drilling through
    the salt, the well is drilled at one side of the dome and is then deviated around and underneath the overhanging cap.

* ###### Shoreline Drilling
    In the case where a reservoir lies offshore but quite close to land, the mose economical way to exploit the reservoir may be to
    drill directional wells from a land rig on the coast.Although it is not a new concept, one type of directional drilling, horiztonal 
    drilling is the fastest growing branch of drilling with major advances occuring in tools and techniques.

#   Depth Reference and Geographical Reference Systems
   The trajectory of a deviated well must be carefully planned so that the most efficient trajectory is used to drill between the rig
   and target location and ensure that the well is drilled for the least amount of money possible. When planning, and subsequently
   drilling the well, the position of all points along the wellpath and therefore the trajectory of the well must be considered in
   three dimensions.This means that the position of all points on the trajectory must be expessed with respect to a three dimensional
   reference system.The three dimensional system that is used to define the position of a particular point along the wellpath is:
    
   *   The vertical depth of the point below a particular reference point
   *   The horizontal distance traversed from the wellhead inn a Northerly direction
   *   The distance traversed from the well head in an Easterly direction
    
   The depth of a particular point in the wellpath is expressed in feet(or metres) vertically below a reference(datum) point and the
   Northerly and Easterly displacement of the point is expressed in feet(or metres) horizontally from the wellhead

##   Depth Reference Systems
   There are a number of datum systems used in the depth reference systems. The datum systems which are most widely used are
    
   * Mean Sea Level,MSL
   * Rotary Table Elevation,RTE
   * 20 Wellhead housing

   The Mean Sea Level,MSL is a permanent, national and well documented datum whereas datum such as the Rotary Table Elevation,RTE 
   only exists when the drilling rig is on site.The top of the 20 Wellhead Housing is only available when the wellhead  housing
   has been installed and will be removed when the well is abandoned. Hence,since the only permanent datum is the MSL,
   (the rig will be removed and the wellhead mya be removed on abandonment) the distance between the MSL and the rotary table
   on the drillfloor and the MSL and the wellhead housing must be measured and recorded carefully on the well survey documents.
   The elevation of the rotary table above the MSL will be measured when the drilling rig is placed over the drilling location.

   The depths of the formations to be penetrated are generally referenced by the geologist and reservoir engineers, to MSL since
   the Rotary Table Elevation will not be known until the driling rig is in place.In most drilling operations the Rotary Table
   Elevation(RTE) is used as the working depth reference since it is relatively simple, for the driller for instance, to measure
   depths relative to this point. The elevation of the (RTE) is also referred to as Derrick Floor ELevation(DFE).Depths measured
   from these references are often called depths below rotary table(BRT) or below derrick floor(BDF).The top of the kelly bushing
   is also used as a datum fo depth measurements, in this case the depths are referred to as depths below rotary kelly bushing(RKB)

   The depth of any point in the wellpath can be expressed in terms of the Along Hole Depth(AHD) and the True Vertical Depth(TVD) of
   the point below the reference datum. The AHD is the depth of a point from the surface reference point,measured along the trajectory
   of the borehole. Whereas the TVD is the vertical depth of the point below the reference point. The AHD will therefore always be
   greater than the TVD in a deviated well. Since there is no direct way of measuring the TVD, it must be calculated from the 
   information gathered when surveying the well.

###### Parameters Defining the WellPath
   There are three specific parameters which must be considered when planning trajectories of wells.These parameters combine to 
   define the trajectory of the well and are the
    
    * Kick-Off Point
    * Buildup and Drop off Rate
    * Tangent Angle of the Well

## The Kickoff Point(KOP)
   The kick off point is the along hole measured depth at which a change in inclination of the well is initiated and the well is 
   orientation in a particular direction(in terms of North,South,East and West). In general the most distant targets have the shallowest
   KOPs in order to reduce the incliation of the tangent section of the well.It is generally easier to kick of a well in shallow
   formations than in deep formations.The kick-off point should be initiated in formations which are stable and not likely to cause
   drilling problems, such as unconsolidated clays.

## Buildup Rate(BUR) and Drop Off Rate(DOR)
   The build up rate and drop off rate(in degrees of inclination)are the rates at which the well deviates from the vertical
   (usually measured in degrees per 100ft drilled).The build-up rate is chosen on the basis of drilling experience in the location
   and the tools available, but rates between 1 degree and 3 degree per 100ft of hole drilled are most common in conventional wells.
   Since the build up and drop off rates are constant, these sections of the well, by definition, form the arc of a circle.Buildup
   rates in excess of 3 degrees per 100ft are termed doglegs when drilling conventional deviated wells with conventional drilling
   equipment.The build up rate is often termed the dogleg severity.

## Tangent(or Drift) Angle
   The tangent angle(or drift angle) is the inclination(in degrees from the vertical) of the long straight section of the well is
   termed the tangent section because it forms a tangent to the arc formed by the build up section of the well.The tangent angle will
   generally be between 10 and 60 degrees since it is difficult to control the trajectory of the well at angles below 10 degrees and
   it is difficult to run wireline tools into wells at anglesof greater than 60 degrees.

#   Defining the Points on the Wellpath
   Having fixed the target and the rig position, the next stage is to plan the geometrical profile of the well to reach the target.
   The most common well trajectory is the build and hold profile, which consists of 3-sections vertical, buildup and tangent.The 
   trajectory of the wellbore can be plotted when the following points have been defined:

    *   KOP --(selected by designer)
    *   TVD[True vertical depth] and horizontal displacement of the end of the build up section.
    *   TVD and horizontal displacement of the target(defined by the position of the righ and target).

Since the driller will only be able to determine the along hole depth of the well the following information will also be required.

    * AHD of the KOP(same as TVD of KOP)
    * Build up rate for the build up section(selected by the designer)
    * Direction in which the well is to be drilled after the KOP in degees from the
        - North(defined by psitoin of the rig target),
        - AHD at which the build up stops and the tangent section commences and 
        - AHD of the target.

These depths and distances can be defined by a simple geometrical analysis of the well trajectory.

## npm package for directional drilling problems

Interface:

    build_UpAngleFxn(): number;

    horizontal_DisplacementFxn(): number;

    total_AngleYFxn(): number;

    total_AngleXplusYFxn(): number;

    measured_DepthAtEndOfBuildSectionFxn(): number;

    trueVertical_DepthAtEndOfBuildFxn(): number;

    horizontal_DeviationAtEndOfBuildUpFxn(): number;
    
    total_MeasuredDepthFxn(): number;

    toDegreesFxn(radians: number): number;

    toRadiansFxn(degrees: number): number;
    
    
## using Classes and Methods

   # TYPESCRIPT
    import { DirectionalDrilling } from "ddrilling";
    import { InitalizeDrillingData } from "ddrilling";

    const inputData : InitalizeDrillingData = {
        buildUpAngle: 2,
        horizontalDisplacement: 3000,
        kickOffPoint: 2000,
        trueVerticalDepth: 10000
    }

    const drilling = new DirectionalDrilling(inputData);
    console.log(drilling.computationalResult());

    OUTPUT

    Directional Drilling Compuational Results
    -------------------------------------------
        Radius      => 2864.7889756541163
        Angle X     => 0.9682854378192228
        Angle Y     => 20.980333796791758
        Angle (X+Y) => 21.94861923461098
        Measured Depth At the End of Build => 3097.430961730549
        Horizontal Deviation at End of Build => 207.64158017340878
        Total Measured Depth => 10568.12478132156

    
   # JAVASCRIPT
   
    var ddrilling = require('ddrilling');

    let data = {
        buildUpAngle: 2,
        horizontalDisplacement: 3000,
        kickOffPoint: 2000,
        trueVerticalDepth: 10000
    }

    var drilling = new ddrilling.DirectionalDrilling(data);
    console.log(drilling.computationalResult())

    OUTPUT

    Directional Drilling Compuational Results
    -------------------------------------------
        Radius      => 2864.7889756541163
        Angle X     => 0.9682854378192228
        Angle Y     => 20.980333796791758
        Angle (X+Y) => 21.94861923461098
        Measured Depth At the End of Build => 3097.430961730549
        Horizontal Deviation at End of Build => 207.64158017340878
        Total Measured Depth => 10568.12478132156
    
   
###### sample Question used for the above implementation

    Given a planning procedure for the build and hold trajectory calculate the drift angle and the true vertical depth,
    given the following basic data:

    Basic Data
    Kick Off Point = 2000
    True Vertical depth = 10000
    Horizontal Displacement from Target = 3000
    BuildUpRate = 2degrees/100ft
